BLE_UUID_SUFFIX = "6085-4726-be45-040c957391b5"
PLEJD_SERVICE = f"31ba0001-{BLE_UUID_SUFFIX}"
PLEJD_LIGHTLEVEL = f"31ba0003-{BLE_UUID_SUFFIX}"
PLEJD_DATA = f"31ba0004-{BLE_UUID_SUFFIX}"
PLEJD_LASTDATA = f"31ba0005-{BLE_UUID_SUFFIX}"
PLEJD_AUTH = f"31ba0009-{BLE_UUID_SUFFIX}"
PLEJD_PING = f"31ba000a-{BLE_UUID_SUFFIX}"
